#include <omp.h>
#include <math.h>

#include "ConjuntoNumeros.h"
#include "/home/marcelo/NetBeansProjects/Biblioteca/MergeSort.h"

#define ITERMAX 10000
#define SAMAX 10000

/*Contrutor recebe o tamanho para alocar memória*/
ConjuntoNumeros::ConjuntoNumeros(int tam) : Lista(tam){
//    this->tam = tam;
    this->conjunto = (int *)malloc(this->tam *sizeof(int *));
    this->saving = (int *)malloc(this->tam *sizeof(int *));
}


/*Definir o numero de partições será feira*/
void ConjuntoNumeros::SetNumParticao(int numParticao){
    this->numParticao = numParticao;
    /*Contem a soma de cada particao*/
    this->soma = (int *)malloc(this->numParticao *sizeof(int *));
}

int ConjuntoNumeros::GetNumParticao(){
    return this->numParticao;
}

/*Recuperar um valor do vetor saving*/
int ConjuntoNumeros::GetValorSaving(int indice){
    return this->saving[indice];
}

/*Setar um valor do vetor saving*/
void ConjuntoNumeros::SetValorSaving(int indice, int valor){
    this->saving[indice] = valor;
}

/*Metodo que imprime os valores do saving*/
void ConjuntoNumeros::ImprimeSaving (){
    cout << "Saving:" << endl;
    for(int i=0; i<this->GetTamanho(); i++){
        cout << this->saving[i] << endl;
    }
    cout << endl;
}

int ConjuntoNumeros::GetMenorSoma(){
    /*Considera que o menor é o indice 0*/
    int menor =  0;
    /*Percorre o vetor soma para encontrar o menor*/
    for (int i = 1; i < this->numParticao; i++) {
        if(this->soma[i] < this->soma[menor])
            menor = i;
    }
    /*Retorna o indice do menor valor*/
    return menor;
}

void ConjuntoNumeros::ConstroiSolucaoGulosa(){
    int menor;

    /*Insere pelo menos um item em cada particao*/
    for (int i = 0; i < this->GetNumParticao(); i++) {
        /*i+1 porque o indice di vetor começa do zero*/
        this->SetValorSaving(i, i+1);
        /*o indice i é o indice que identifica o item*/
        this->soma[i] = this->GetValorConjunto(i);
    }

    /*Recupera o indice da particao com a menor soma*/
    menor = this->GetMenorSoma();

    /*Insere os itens da lista na particao com menor soma*/
    for (int i = this->GetNumParticao(); i < this->GetTamanho(); i++) {

        this->SetValorSaving(i, menor+1);
        this->soma[menor] += this->GetValorConjunto(i);

        /*Atualiza o valor menor soma*/
        menor = this->GetMenorSoma();
    }

    this->AtualizaSoma();
}


void ConjuntoNumeros::ConstroiSolucaoAleatoria(){
    int particao;
    
    srand(time(NULL));

    /*Insere um item em cada particao*/
    for (int i = 0; i < this->GetTamanho(); i++) {
        particao = rand()%this->GetNumParticao();
        /*particao+1 porque o indice do vetor começa do zero e a partiçao começa em 1*/
        this->SetValorSaving(i, particao+1);
        /*o indice i é o indice que identifica o item*/
        this->soma[particao] += this->GetValorConjunto(i);

    }

    this->AtualizaSoma();
}


void ConjuntoNumeros::MovimentoSubstituicao(int i, int particao){
    this->saving[i] = particao;
    
    /*TODO*/
    /*Melhorar para atualizar apenas a soma alterada*/
    this->AtualizaSoma();
}

void ConjuntoNumeros::MovimentoTroca(int i, int j){
    int aux;
    aux = this->saving[i];
    this->saving[i] = this->saving[j];
    this->saving[j] = aux;
    
    /*TODO*/
    /*Melhorar para atualizar apenas a soma alterada*/
    this->AtualizaSoma();
}

int ConjuntoNumeros::AvaliaFuncaoObjetivo(){

    //this->AtualizaSoma();
    
    int minSoma = this->soma[0], maxSoma = this->soma[0];
    for (int i = 1; i < this->GetNumParticao(); i++) {
        if(this->soma[i] > maxSoma)
            maxSoma = this->soma[i];
        else if(this->soma[i] < minSoma)
            minSoma = this->soma[i];
    }
    return maxSoma - minSoma;
}

void ConjuntoNumeros::ImprimeParticionamento(){

    /*Imprimir os itens de todas as partições*/
    for (int i = 0; i < this->GetNumParticao(); i++) {
        cout << "Conjunto " << i+1 << " : ";
        for (int j = 0; j < this->GetTamanho(); j++) {
            if(this->GetValorSaving(j) == i+1)
                cout << this->GetValorConjunto(j) << " ";
        }
        cout << endl;
    }
}

void ConjuntoNumeros::AtualizaSoma(){
    
    for (int i = 0; i < this->GetNumParticao(); i++) {
        this->soma[i] = 0;
    }


    for (int i = 0; i < this->GetTamanho(); i++) {
        this->soma[this->GetValorSaving(i)-1] += this->GetValorConjunto(i);
    }
}

void ConjuntoNumeros::ImprimeSoma(){
    cout << "Soma dos itens" << endl;
    for (int i = 0; i < this->GetNumParticao(); i++) {
        cout << "Soma do conjunto " << i+1 << ": " << this->soma[i] << endl;
    }
    
    cout << "Funcao Objetivo: " << this->AvaliaFuncaoObjetivo() << endl;
}


/*-------------------------------Best Improvement---------------------------------------------*/
/*Encontrar melhor vizinho de melhora para troca*/
void ConjuntoNumeros::MelhorVizinhoTrocaMelhora(int &melhorTrocaI, int &melhorTrocaJ, int &melhorTrocaFO){
    int foVizinho;

    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
    if(melhorTrocaFO == 0)
        return;

    /*Realiza movimentos de troca*/
    for (int i = 0; i < this->GetTamanho()-1; i++) {
        for (int j = i+1; j < this->GetTamanho(); j++) {

            /*Comparar se a troca é feita com um elemento do mesmo grupo*/
            if(this->saving[i] != this->saving[j]){

                /*Realiza a troca*/
                this->MovimentoTroca(i, j);
                /*Avalia a troca*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a troca*/
                this->MovimentoTroca(j, i);
            
                /*Caso a função tenha melhorado, guarda i e j*/
                if(foVizinho < melhorTrocaFO){
                    melhorTrocaI = i;
                    melhorTrocaJ = j;
                    /*Atualiza fo*/
                    melhorTrocaFO = foVizinho;
                    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
                    if(melhorTrocaFO == 0)
                        return;
                }
            }
        }
    }
}


void ConjuntoNumeros::MelhorVizinhoSubstMelhora(int &melhorSubstI, int &melhorSubstP, int &melhorSubstFO){
    int foVizinho, pAnterior;

    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
    if(melhorSubstFO == 0)
        return;

    /*Realiza movimentos de troca*/
    for (int i = 0; i < this->GetTamanho(); i++) {
        for (int p = 1; p <= this->GetNumParticao(); p++) {

            /*Comparar se a substituição é feita para o mesmo grupo*/
            if(this->saving[i] != p){

                pAnterior = this->saving[i];

                /*Realiza a substituição*/
                this->MovimentoSubstituicao(i, p);
                /*Avalia a substituição*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a substituição*/
                this->MovimentoSubstituicao(i, pAnterior);

                /*Caso a função tenha melhorado, guarda i e j*/
                if(foVizinho < melhorSubstFO){
                    melhorSubstI = i;
                    melhorSubstP = p;
                    /*Atualiza fo*/
                    melhorSubstFO = foVizinho;
                    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
                    if(melhorSubstFO == 0)
                        return;
                }
            }
        }
    }
}

/*Método da descida*/
void ConjuntoNumeros::BestImprovement(){

    int melhorTrocaI, melhorTrocaJ, melhorTrocaFO, fo, melhorSubstI, melhorSubstP, melhorSubstFO;

    fo = melhorTrocaFO = melhorSubstFO = this->AvaliaFuncaoObjetivo();
    int i = 0;
    
    while(true){
        this->MelhorVizinhoTrocaMelhora(melhorTrocaI, melhorTrocaJ, melhorTrocaFO);
        this->MelhorVizinhoSubstMelhora(melhorSubstI, melhorSubstP, melhorSubstFO);

        if(melhorTrocaFO < melhorSubstFO){
            if(melhorTrocaFO < fo){
                this->MovimentoTroca(melhorTrocaI, melhorTrocaJ);
                fo = this->AvaliaFuncaoObjetivo();
            }
            /*Caso não seja possível melhorar sai o loop.*/
            else{
                break;
            }
        }
        else{
            if(melhorSubstFO < fo){
                this->MovimentoSubstituicao(melhorSubstI, melhorSubstP);
                fo = this->AvaliaFuncaoObjetivo();
            }
            /*Caso não seja possível melhorar sai o loop.*/
            else{
                break;
            }
        }
        i++;
    }
}

/*-------------------------------Fim Best Improvement---------------------------------------------*/

/*-------------------------------First Improvement---------------------------------------------*/

/*Encontrar melhor vizinho para troca*/
void ConjuntoNumeros::PrimeiroMelhorVizinhoTroca(int &melhorTrocaI, int &melhorTrocaJ, int &melhorTrocaFO){
    int foVizinho, *vetAleatorio, tam;

    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
    if(melhorTrocaFO == 0)
        return;

    /*Vetor aleatorio para não melhorar apenas os primeiros elementos*/
    vetAleatorio = (int *)(malloc(this->GetTamanho()*(sizeof(int))));

    tam = this->GetTamanho();
    EmbaralhaVetor(vetAleatorio, tam);

    /*Realiza movimentos de troca*/
    for (int i = 0; i < this->GetTamanho()-1; i++) {
        for (int j = i+1; j < this->GetTamanho(); j++) {

            /*Comparar se a troca é feita com um elemento do mesmo grupo*/
            if(this->saving[vetAleatorio[i]] != this->saving[vetAleatorio[j]]){

                /*Realiza a troca*/
                this->MovimentoTroca(vetAleatorio[i], vetAleatorio[j]);
                /*Avalia a troca*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a troca*/
                this->MovimentoTroca(vetAleatorio[j], vetAleatorio[i]);

                /*Caso a função tenha melhorado, guarda i e j, e retorna*/
                if(foVizinho < melhorTrocaFO){
                    melhorTrocaI = vetAleatorio[i];
                    melhorTrocaJ = vetAleatorio[j];
                    /*Atualiza fo*/
                    melhorTrocaFO = foVizinho;
                    return;
                }
            }
        }
    }
}


void ConjuntoNumeros::PrimeiroMelhorVizinhoSubst(int &melhorSubstI, int &melhorSubstP, int &melhorSubstFO){
    int foVizinho, pAnterior, *vetAleatorio, tam;

    /*Caso a função objetivo seja igual a zero encontrou uma solução ótima.*/
    if(melhorSubstFO == 0)
        return;

    /*Vetor aleatorio para não melhorar apenas os primeiros elementos*/
    vetAleatorio = (int *)(malloc(this->GetTamanho()*(sizeof(int))));

    tam = this->GetTamanho();
    EmbaralhaVetor(vetAleatorio, tam);

    /*Realiza movimentos de troca*/
    for (int i = 0; i < this->GetTamanho(); i++) {
        for (int p = i+1; p < this->GetNumParticao(); p++) {

            /*Comparar se a substituição é feita para o mesmo grupo*/
            if(this->saving[vetAleatorio[i]] != p){

                pAnterior = this->saving[vetAleatorio[i]];

                /*Realiza a substituição*/
                this->MovimentoSubstituicao(vetAleatorio[i], p);
                /*Avalia a substituição*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a substituição*/
                this->MovimentoSubstituicao(vetAleatorio[i], pAnterior);

                /*Caso a função tenha melhorado, guarda i e j, e retorna*/
                if(foVizinho < melhorSubstFO){
                    melhorSubstI = vetAleatorio[i];
                    melhorSubstP = p;
                    /*Atualiza fo*/
                    melhorSubstFO = foVizinho;
                    return;
                }
            }
        }
    }
}

/*Método da descida com primeira melhora*/
void ConjuntoNumeros::FirstImprovement(){
    int melhorTrocaI, melhorTrocaJ, melhorTrocaFO, fo, melhorSubstI, melhorSubstP, melhorSubstFO, op;
    srand(time(NULL));
    int iter = 0;

    fo = melhorTrocaFO = melhorSubstFO = this->AvaliaFuncaoObjetivo();

    while(iter < ITERMAX){
        /*ate 80 um tipo*/
        op = rand()%2;
        if(op == 0){
            this->PrimeiroMelhorVizinhoTroca(melhorTrocaI, melhorTrocaJ, melhorTrocaFO);
            this->PrimeiroMelhorVizinhoSubst(melhorSubstI, melhorSubstP, melhorSubstFO);
        }
        else{
            this->PrimeiroMelhorVizinhoSubst(melhorSubstI, melhorSubstP, melhorSubstFO);
            this->PrimeiroMelhorVizinhoTroca(melhorTrocaI, melhorTrocaJ, melhorTrocaFO);
        }
        
        if(melhorTrocaFO < melhorSubstFO){
            if(melhorTrocaFO < fo){
                this->MovimentoTroca(melhorTrocaI, melhorTrocaJ);
                fo = this->AvaliaFuncaoObjetivo();
            }
            /*Caso não seja possível melhorar sai o loop.*/
            else{
                iter++;
            }
        }
        else{
            if(melhorSubstFO < fo){
                this->MovimentoSubstituicao(melhorSubstI, melhorSubstP);
                fo = this->AvaliaFuncaoObjetivo();
            }
            /*Caso não seja possível melhorar sai o loop.*/
            else{
                iter++;
            }
        }
    }
}

/*------------------------------- Fim First Improvement---------------------------------------------*/


/*-------------------------------Random Improvement---------------------------------------------*/

void ConjuntoNumeros::RandomImprovement(){
    int iter = 0;
    int op, i, j, fo, foVizinho, particaoAnterior;

    fo = this->AvaliaFuncaoObjetivo();

    do{
        op = rand()%2;
        /*Troca*/
        if(op == 0){
            i = rand()%this->GetTamanho();
            do{
                j = rand()%this->GetTamanho();
            }while(this->saving[i] != this->saving[j]);
            /*Faz a troca*/
            this->MovimentoTroca(i, j);
            foVizinho = this->AvaliaFuncaoObjetivo();
            if(foVizinho < fo){
                fo = foVizinho;
                iter = 0;
            }
            else{
                /*Desfaz a troca*/
                this->MovimentoTroca(j, i);
                iter ++;
            }
        }
        /*Substituição*/
        else{
            /*i é o indice*/
            i = rand()%this->GetTamanho();
            do{
                /*j é o conjunto*/
                //srand(time(NULL));
                j = rand()%this->GetNumParticao();
                j++;
            }while(this->saving[i] == j);
            /*Armazena o conjunto anterior, caso necessite desfazer a operação*/
            particaoAnterior = this->saving[i];
            this->MovimentoSubstituicao(i, j);
            foVizinho = this->AvaliaFuncaoObjetivo();
            if(foVizinho < fo){
                fo = foVizinho;
                iter = 0;
            }
            else{
                /*Desfaz a troca*/
                this->MovimentoSubstituicao(i, particaoAnterior);
                iter ++;
            }
        }
    }while(iter < ITERMAX);
}

/*-------------------------------Fim Random Improvement---------------------------------------------*/


/*-------------------------------MultiStart---------------------------------------------*/

void ConjuntoNumeros::MultiStart(){
    int fo, foStart;
    int *saving;
    
    /*Vetor para armazenar a melhor solução temporariamente*/
    saving = (int *)malloc(this->GetTamanho() *sizeof(int *));
    
    this->ConstroiSolucaoAleatoria();
    fo = foStart = this->AvaliaFuncaoObjetivo();
    
    /*Guarda a solução*/
    CopiaVetor(this->saving, saving, this->GetTamanho());
    
    
    for (int i = 0; i < ITERMAX; i++) {   
        this->BestImprovement();
        fo = this->AvaliaFuncaoObjetivo();
        
        if(fo < foStart){
            /*Guarda a solução*/
            CopiaVetor(this->saving, saving, this->GetTamanho());
            foStart = fo;
        }
        this->ConstroiSolucaoAleatoria();
    }
    
    /*Recupera a melhor solução salva solução*/
    CopiaVetor(saving, this->saving, this->GetTamanho());
    /*O vetor soma esta com valores da ultima iteração feita, portanto, 
     * é necessário fazer a atualização*/
    this->AtualizaSoma();
}

/*-------------------------------Fim MultiStart---------------------------------------------*/


/*-------------------------------BuscaTabu---------------------------------------------*/


/*Encontrar melhor vizinho para troca*/
void ConjuntoNumeros::MelhorVizinhoTroca(int &melhorTrocaI, int &melhorTrocaJ, int &foMelhorTroca){
    int foVizinho;
    
    int k=0;
    
    while(k < this->GetTamanho()-1){
        k++;
        if(this->saving[0] != this->saving[k])
            break;
    }
    
    /*Realiza a troca*/
    this->MovimentoTroca(0, k);
    /*Avalia a troca*/
    foMelhorTroca = this->AvaliaFuncaoObjetivo();
    /*Desfaz a troca*/
    this->MovimentoTroca(k, 0);

    
    /*Realiza movimentos de troca*/
    for (int i=0; i < this->GetTamanho()-1; i++) {
        for (int j = i+1; j < this->GetTamanho(); j++) {

            /*Comparar se a troca é feita com um elemento do mesmo grupo*/
            if(this->saving[i] != this->saving[j]){

                /*Realiza a troca*/
                this->MovimentoTroca(i, j);
                /*Avalia a troca*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a troca*/
                this->MovimentoTroca(j, i);
            
                /*Caso a função seja a melhor encontrada, guarda i e j*/
                if(foVizinho < foMelhorTroca){
                    melhorTrocaI = i;
                    melhorTrocaJ = j;
                    /*Atualiza fo*/
                    foMelhorTroca = foVizinho;
                }
            }
        }
    }
}


void ConjuntoNumeros::MelhorVizinhoSubst(int &melhorSubstI, int &melhorSubstP, int &foMelhorSubst){
    
    int foVizinho, pAnterior;
    
    int k=0;
    
    while(k < this->GetNumParticao()){
        k++;
        if(this->saving[0] != k)
            break;
            
    }
    
    /*Realiza a substituição*/
    this->MovimentoSubstituicao(0, k);
    /*Avalia a substituição*/
    foMelhorSubst = this->AvaliaFuncaoObjetivo();
    /*Desfaz a substituição*/
    this->MovimentoSubstituicao(k, 0);

    
    /*Realiza movimentos de substituição*/
    for (int i = 0; i < this->GetTamanho(); i++) {
        for (int p = 0; p < this->GetNumParticao(); p++) {

            /*Comparar se a substituição é feita para o mesmo grupo*/
            if(this->saving[i] != p){

                pAnterior = this->saving[i];

                /*Realiza a substituição*/
                this->MovimentoSubstituicao(i, p);
                /*Avalia a substituição*/
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a substituição*/
                this->MovimentoSubstituicao(i, pAnterior);

                /*Caso a função tenha melhorado, guarda i e j*/
                if(foVizinho < foMelhorSubst){
                    melhorSubstI = i;
                    melhorSubstP = p;
                    /*Atualiza fo*/
                    foMelhorSubst = foVizinho;
                }
            }
        }
    }
}



void ConjuntoNumeros::BuscaTabu(){
//    int fo, foVizinho, melhorIter, iter;
//    int *saving;
//    int tabu[this->GetNumParticao()][this->GetTamanho()];
//    
//    /*Vetor para armazenar a melhor solução temporariamente*/
//    saving = (int *)malloc(this->GetTamanho() *sizeof(int *));
//    
//    for (int i = 0; i < this->GetNumParticao(); i++) {
//        for (int j = 0; j < this->GetTamanho(); j++) {
//            tabu[i][j] = 0;
//        }
//
//    }
    
    int foMelhorVizinho, melhorTrocaI, melhorTrocaJ, melhorTrocaFO, fo, melhorSubstI, melhorSubstP, melhorSubstFO;
    
    //int iterMax = 0;

    fo = melhorTrocaFO = melhorSubstFO = this->AvaliaFuncaoObjetivo();
    
    /*Vetor para armazenar a melhor solução temporariamente*/
    int *saving = (int *)malloc(this->GetTamanho() *sizeof(int *));
    
    /*Guarda a solução*/
    CopiaVetor(this->saving, saving, this->GetTamanho());

    for (int i = 0; i < ITERMAX; i++) {
        
        melhorTrocaFO = melhorSubstFO = 0;

        this->MelhorVizinhoTroca(melhorTrocaI, melhorTrocaJ, melhorTrocaFO);
        this->MelhorVizinhoSubst(melhorSubstI, melhorSubstP, melhorSubstFO);
        if(melhorTrocaFO < melhorSubstFO){
            this->MovimentoTroca(melhorTrocaI, melhorTrocaJ);
        }
        else{
            this->MovimentoSubstituicao(melhorSubstI, melhorSubstP);
        }
        
        foMelhorVizinho = this->AvaliaFuncaoObjetivo();
        
        
        if(foMelhorVizinho < fo){
            /*Guarda a solução*/
            CopiaVetor(this->saving, saving, this->GetTamanho());
        }
    }
    
    /*Recupera a melhor solução salva solução*/
    CopiaVetor(saving, this->saving, this->GetTamanho());
    /*O vetor soma esta com valores da ultima iteração feita, portanto, 
     * é necessário fazer a atualização*/
    this->AtualizaSoma();

    free(saving);
}

/*-------------------------------Fim BuscaTabu---------------------------------------------*/

/*-------------------------------Grasp-------------------------------------------------*/

/*Alpha de 0 a 100*/
int ConjuntoNumeros::GetParticaoListaRestrita(int alpha){
    
    int particao;
    int a;
    
    
    int **aux;
    aux = (int **)malloc(2*sizeof(int));
    aux[0] = (int *)malloc(this->numParticao*sizeof(int));
    aux[1] = (int *)malloc(this->numParticao*sizeof(int));
    
    for (int i = 0; i < this->numParticao; i++) {
        aux[0][i] = this->soma[i];
        aux[1][i] = i;
    }
    
    /*Ordenar aux para selecionar os primeiros*/
    int i, j, chave, temp;
    for (i = 1; i < this->numParticao; i++) {
        chave = aux[0][i];
        temp = aux[1][i];
        j=i-1;
        while(j>=0 && aux[0][j]>chave){
            aux[0][j+1] = aux[0][j];
            aux[1][j+1] = aux[1][j];
            j=j-1;
        }
        aux[0][j+1]=chave;
        aux[1][j+1]=temp;
        
    }

    
    /*Selecionar os primeiros, conforme alpha*/
    srand(time(NULL));
    
    /*Seleciona a particao aleatoriamente, ate o limite alpha*/
    a = alpha*this->numParticao/100;
    if(a)
        particao = rand()%(a);
    else
        particao = 0;
    
    /*Retorna a particao selecionada*/
    return aux[1][particao];
}


/*Alpha de 0 a 100*/
int ConjuntoNumeros::GetIndiceListaRestrita(int alpha, int *listaRestrita, int n){
    
    int indice;
    int a;
        
    /*Selecionar os primeiros, conforme alpha*/
    srand(time(NULL)+n);
    
    /*Seleciona a particao aleatoriamente, ate o limite alpha*/
    a = alpha*n/100;
    if(a)
        indice = rand()%(a);
    else
        indice = 0;
    
    int i;
    for (i = 0; i < n; i++) {
        if(listaRestrita[i] == 1)
            continue;
        if(i == indice)
            break;
    }

    
    /*Retorna a particao selecionada*/
    return i;
}

void ConjuntoNumeros::ConstrucaoGrasp(int alpha){
    
    /*aleatorio*/
    int particao, indice, n;
    
    /*Armazena quais numeros foram incluidos, 0 nao incluido, 1 incluido*/
    int *listaRestrita;
    listaRestrita = (int *)malloc(sizeof(int)*this->tam);
    
    for(int i=0; i<this->tam; i++){
        listaRestrita[i] = 0;
    }
    n = this->tam-1;

    /*Insere um item em cada particao*/
    for (int i = 0; i < this->GetTamanho(); i++) {
        particao = this->GetParticaoListaRestrita(alpha);
        
        indice = this->GetIndiceListaRestrita(alpha, listaRestrita, n);
        listaRestrita[indice] = 1;
        n--;
        
        /*particao+1 porque o indice do vetor começa do zero e a partiçao começa em 1*/
        this->SetValorSaving(indice, particao+1);
        
        /*o indice i é o indice que identifica o item*/
        this->soma[particao] += this->GetValorConjunto(indice);

    }

    this->AtualizaSoma();
    
}

void ConjuntoNumeros::Grasp(){
    
    /*Vetor para armazenar a melhor solução temporariamente*/
    int *save = (int *)malloc(this->GetTamanho() *sizeof(int *));
    
    /*Guarda a solução*/
    CopiaVetor(this->saving, save, this->GetTamanho());
    
    int melhorFo = this->AvaliaFuncaoObjetivo();
    int fo;
    
    //MensagemFOInicial(melhorFo);
    
    int alpha = 20;
    
   
    for (int i = 0; i < ITERMAX; i++) {
        
        this->ConstrucaoGrasp(alpha);
        this->BestImprovement();
        //this->RandomImprovement();
        
        fo = this->AvaliaFuncaoObjetivo();
        
        /*Se encontrou um solucao melhor*/
        if(fo < melhorFo){
            melhorFo = fo;
            /*Guarda a solução*/
            CopiaVetor(this->saving, save, this->GetTamanho());
            MensagemMelhoraFO(i, melhorFo);
        }
        
    }
    
    /*Se a melhor solucao e melhor que a atual*/
    if(melhorFo < this->AvaliaFuncaoObjetivo()){
        /*Recupera a melhor solução salva solução*/
        CopiaVetor(save, this->saving, this->GetTamanho());
        /*O vetor soma esta com valores da ultima iteração feita, portanto, 
         * é necessário fazer a atualização*/
        this->AtualizaSoma();
    }

    free(save);
}
/*-------------------------------Fim Grasp---------------------------------------------*/

/*---------------------------Simulated Anneling----------------------------------------*/
void ConjuntoNumeros::SimulatedAnnealing(){
    double tempInicial, tempFinal, tempAtual, x, alpha;
    int iter, op, i, j, fo, melhorFo, foVizinho, delta, particaoAnterior, tempSemMelhora;
    
    tempInicial = this->TemperaturaInicialSA();
    tempFinal = 1;
    alpha = 0.998;
    fo = melhorFo = this->AvaliaFuncaoObjetivo();
    tempAtual = tempInicial;
    cout << "Mehor FO: " << melhorFo << "\t Temperatura: " << tempAtual << endl;
    
    /*Vetor para armazenar a melhor solução temporariamente*/
    int *save = (int *)malloc(this->GetTamanho() *sizeof(int *));
    
    /*Guarda a solução*/
    CopiaVetor(this->saving, save, this->GetTamanho());
    
    tempSemMelhora = 0;
    
    while (tempAtual > tempFinal && tempSemMelhora < SAMAX){

        iter = 0;
        while (iter < SAMAX){
            iter++;
            op = rand()%2;
            /*Troca*/
            if(op == 0){
                i = rand()%this->GetTamanho();
                do{
                    j = rand()%this->GetTamanho();
                }while(this->saving[i] != this->saving[j]);
                /*Faz a troca*/
                this->MovimentoTroca(i, j);
                foVizinho = this->AvaliaFuncaoObjetivo();
                
                delta = foVizinho - fo;
                /*Caso tenha melhorado*/
                if(delta < 0){
                    fo = foVizinho;
                    if(fo<melhorFo){
                        melhorFo = fo;
                        
                        CopiaVetor(this->saving, save, this->GetTamanho());
                        
                        cout << "Mehor FO: " << melhorFo << "\t Temperatura: " << tempAtual << endl;
                        tempSemMelhora = 0;
                    }
                }
                else{
                    x = Randomico(0,1);
                    if(x < exp(-delta/tempAtual)){
                        fo = foVizinho;
                    }
                    else{
                        /*Desfaz a troca*/
                        this->MovimentoTroca(j, i);
                        tempSemMelhora++;
                    }
                }
                
            }
            /*Substituição*/
            else{
                /*i é o indice*/
                i = rand()%this->GetTamanho();
                do{
                    /*j é o conjunto*/
                    //srand(time(NULL));
                    j = rand()%this->GetNumParticao();
                    /*As particoes estao sendo armazenas a partir de 1*/
                    j++;
                }while(this->saving[i] == j);
                /*Armazena o conjunto anterior, caso necessite desfazer a operação*/
                particaoAnterior = this->saving[i];
                /*Faz a substituicao*/
                this->MovimentoSubstituicao(i, j);
                foVizinho = this->AvaliaFuncaoObjetivo();
                
                delta = foVizinho - fo;
                /*Caso tenha melhorado*/
                if(delta < 0){
                    fo = foVizinho;
                    if(fo<melhorFo){
                        melhorFo = fo;
                        
                        CopiaVetor(this->saving, save, this->GetTamanho());
                        
                        cout << "Mehor FO: " << melhorFo << "\t Temperatura: " << tempAtual << endl;
                        tempSemMelhora = 0;
                    }
                }
                else{
                    x = Randomico(0,1);
                    //cout << "exp(-delta/temperatura): " << exp(-delta/temperatura) << endl;
                    if(x < exp(-delta/tempAtual)){
                        fo = foVizinho;
                    }
                    else{
                        /*Desfaz a substituicao*/
                        this->MovimentoSubstituicao(i, particaoAnterior);
                        tempSemMelhora++;
                    }
                }
            }
        }
        tempAtual = tempAtual * alpha;
    }
    
    /*Se a melhor solucao e melhor que a atual*/
    if(melhorFo < this->AvaliaFuncaoObjetivo()){
        /*Recupera a melhor solução salva solução*/
        CopiaVetor(save, this->saving, this->GetTamanho());
        /*O vetor soma esta com valores da ultima iteração feita, portanto, 
         * é necessário fazer a atualização*/
        this->AtualizaSoma();
    }

    free(save);
}

double ConjuntoNumeros::TemperaturaInicialSA(){
    double temperatura, x, beta, gamma;
    temperatura = 10;
    
    beta = 1.1; // taxa de reaquecimento
    gamma = 0.95;// taxa de resfriamento
    
    int op, i, j, fo, foVizinho, particaoAnterior, aceitos, iter, delta;

    fo = this->AvaliaFuncaoObjetivo();
    
    while(true){
        aceitos = 0;
        iter = 0;
        while(iter < SAMAX ){
            iter++;
            op = rand()%2;
            /*Troca*/
            if(op == 0){
                i = rand()%this->GetTamanho();
                do{
                    j = rand()%this->GetTamanho();
                }while(this->saving[i] != this->saving[j]);
                /*Faz a troca*/
                this->MovimentoTroca(i, j);
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a troca*/
                this->MovimentoTroca(j, i);
                delta = foVizinho - fo;
                /*Caso tenha melhorado*/
                if(delta < 0){
                    aceitos++;
                }
                else{
                    //x = rand()%1;
                    x = Randomico(0,1);
                    if(x < exp(-delta/temperatura)){
                        aceitos++;
                    }
                }
                
            }
            /*Substituição*/
            else{
                /*i é o indice*/
                i = rand()%this->GetTamanho();
                do{
                    /*j é o conjunto*/
                    //srand(time(NULL));
                    j = rand()%this->GetNumParticao();
                }while(this->saving[i] == j);
                /*Armazena o conjunto anterior, caso necessite desfazer a operação*/
                particaoAnterior = this->saving[i];
                /*Faz a substituicao*/
                this->MovimentoSubstituicao(i, j);
                foVizinho = this->AvaliaFuncaoObjetivo();
                /*Desfaz a substituicao*/
                this->MovimentoSubstituicao(i, particaoAnterior);
                delta = foVizinho - fo;
                if(delta < 0){
                    aceitos++;
                }
                else{
                    x = Randomico(0,1);
                    //cout << "exp(-delta/temperatura): " << exp(-delta/temperatura) << endl;
                    if(x < exp(-delta/temperatura)){
                        aceitos++;
                    }
                }
            }
        }
        if (aceitos < gamma*SAMAX)
            temperatura = beta * temperatura;
        else
            break;
    }
    
    return temperatura;
}
/*-------------------------Fim Simulated Anneling--------------------------------------*/


/*-------------------------------Embaralha Vetor-------------------------------------------*/


void EmbaralhaVetor(int *vetAleatorio, int tam){
    int a, b, aux;
    for (int i = 0; i < tam; i++) {
        vetAleatorio[i] = i;
    }

    for (int i = 0; i < tam/2; i++) {
        a = rand()%tam;
        b = rand()%tam;
        aux = vetAleatorio[a];
        vetAleatorio[a] = vetAleatorio[b];
        vetAleatorio[b] = aux;
    }
}

/*-------------------------------Fim Embaralha Vetor-------------------------------------------*/


/*-------------------------------Copia Vetor--------------------------------------------------*/

void CopiaVetor(int *origem, int *destino, int tam){
    for (int i = 0; i < tam; i++) {
        destino[i] = origem[i];
    }
}

/*-------------------------------Fim Copia Vetor-----------------------------------------------*/

void MensagemMelhoraFO(int iteracao, int melhorFo){
    cout << "Melhora na Iteracao: " << iteracao << " FO Atual: " << melhorFo << endl;
}

void MensagemFOInicial(int melhorFo){
    cout << "FO Inicial: " << melhorFo << endl;
}

double Randomico(int inicial, int final){
    double aux;
    aux = rand();
    if(!final)
        aux = 0;
    else
        aux = inicial + aux*(final-inicial)/RAND_MAX;
    return aux;
}